//
//  SearchTextFieldCell.m
//  SearchTextField
//
//  Created by Prachi Gauriar on Mon Mar 24 2003.
//  Copyright (c) 2003 Prachi Gauriar. All rights reserved.
//

#import "STFPopUpButtonCell.h"
#import "SearchTextFieldCell.h"

static const float symbolXOffset = 4.0;
static const float symbolYOffset = 4.0;

static NSImage *cachedLeftImage;
static NSImage *cachedMiddleImage;
static NSImage *cachedRightImage;
static NSImage *cachedPopUpImage;
static NSImage *cachedCancelImage;


@interface SearchTextFieldCell(PrivateMethods)

- (BOOL)_loadImagesIfNecessary;
- (void)_popUpButtonSelectionChanged:(NSNotification *)aNotification;

@end


@implementation SearchTextFieldCell

- (id)initTextCell:(NSString *)aString
{
    [super initTextCell:aString];

    if ([self _loadImagesIfNecessary]) {
        _leftImage = [cachedLeftImage copy];
        _middleImage = [cachedMiddleImage copy];
        _rightImage = [cachedRightImage copy];
        _popUpImage = [cachedPopUpImage copy];
        _cancelImage = [cachedCancelImage copy];

        hasPopUpButton = YES;
        _shouldShowCancelButton = NO;
        _shouldShowSelectedPopUpItem = YES;

        popUpButtonCell = [[STFPopUpButtonCell alloc] initTextCell:@"" pullsDown:YES];
        [popUpButtonCell addItemWithTitle:@""];
        
        [self setEditable:YES];
        [self setScrollable:YES];

        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(_popUpButtonSelectionChanged:)
                                                     name:@"STFPopUpButtonCellSelectionChanged"
                                                   object:popUpButtonCell];
        
        return self;
    } else {
        return nil;
    }
}


- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [popUpButtonCell release];
    
    [_leftImage release];
    [_rightImage release];
    [_middleImage release];
    [_popUpImage release];
    [_cancelImage release];

    [super dealloc];

    return;
}


- (BOOL)hasPopUpButton
{
    return hasPopUpButton;
}


- (void)setHasPopUpButton:(BOOL)aBoolean
{
    hasPopUpButton = aBoolean;

    return;
}


- (STFPopUpButtonCell *)popUpButtonCell
{
    if (hasPopUpButton) {
        return popUpButtonCell;
    } else {
        return nil;
    }
}


- (void)showSelectedPopUpItem:(BOOL)shouldShow
{
    NSString *newTitle = [[self  popUpButtonCell] titleOfSelectedItem];

    if (shouldShow && _shouldShowSelectedPopUpItem) {
        if (hasPopUpButton && newTitle && [[self stringValue] isEqualToString:@""]) {
            [self setTextColor:[NSColor disabledControlTextColor]];
            [self setStringValue:newTitle];
        }
    } else {
        [self setTextColor:[NSColor controlTextColor]];
    }

    _shouldShowSelectedPopUpItem = shouldShow;

    return;
}


- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{    
    NSPoint _leftImageOrigin, _middleImageOrigin, _rightImageOrigin;
    NSPoint _popUpImageOrigin, _cancelImageOrigin;
    float _middleImageWidth;
    BOOL wasShowingFirstResponder = [self showsFirstResponder];

    // Draw the left end of the widget
    _leftImageOrigin = cellFrame.origin;
    _leftImageOrigin.y += [_leftImage size].height;
    [_leftImage compositeToPoint:_leftImageOrigin operation:NSCompositeSourceOver];

    // Draw the right end of the widget
    _rightImageOrigin = cellFrame.origin;
    _rightImageOrigin.x += cellFrame.size.width - [_rightImage size].width;
    _rightImageOrigin.y += [_rightImage size].height;
    [_rightImage compositeToPoint:_rightImageOrigin operation:NSCompositeSourceOver];

    // Draw the middle section
    _middleImageOrigin = cellFrame.origin;
    _middleImageOrigin.x += [_leftImage size].width;
    _middleImageOrigin.y += [_middleImage size].height;
    _middleImageWidth = cellFrame.size.width - ([_leftImage size].width + [_rightImage size].width);
    [_middleImage setSize:NSMakeSize(_middleImageWidth, [_middleImage size].height)];
    [_middleImage compositeToPoint:_middleImageOrigin operation:NSCompositeSourceOver];

    // If we have a popUp button, draw it on the widget
    if (hasPopUpButton) {
        _popUpImageOrigin = cellFrame.origin;
        _popUpImageOrigin.x += symbolXOffset;
        _popUpImageOrigin.y += [_popUpImage size].height + symbolYOffset;
        [_popUpImage compositeToPoint:_popUpImageOrigin operation:NSCompositeSourceOver];
    }

    // If we should show the cancel button, draw the button in the proper rect
    if (_shouldShowCancelButton) {
        _cancelImageOrigin = cellFrame.origin;
        _cancelImageOrigin.x += cellFrame.size.width - [_cancelImage size].width - symbolXOffset;
        _cancelImageOrigin.y += [_cancelImage size].height + symbolYOffset;
        [_cancelImage compositeToPoint:_cancelImageOrigin operation:NSCompositeSourceOver];        
    }

    if ([self controlView] && [[self controlView] isFirstResponder] == NO) {
        [self showSelectedPopUpItem:YES];
    }
    
    
    // Invalidate the focus ring
    [controlView setKeyboardFocusRingNeedsDisplayInRect:cellFrame];

    // Draw a custom focus ring if necessary
    if (wasShowingFirstResponder && [[controlView window] isKeyWindow]) {
        [NSGraphicsContext saveGraphicsState];

        NSSetFocusRingStyle(NSFocusRingOnly);
        [[self fieldOutlinePathForRect:cellFrame] fill];

        [NSGraphicsContext restoreGraphicsState];
    }

    // Draw the text field
    [self setShowsFirstResponder:NO];
    [super drawWithFrame:[self textFieldRectFromRect:cellFrame] inView:controlView];
    [self setShowsFirstResponder:wasShowingFirstResponder];
    
    return;
}


- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj
             delegate:(id)anObject event:(NSEvent *)theEvent
{
    // Make sure that the frame is the text field rect and pass it up to the superclass
    [super editWithFrame:[self textFieldRectFromRect:aRect]
                  inView:controlView
                  editor:textObj
                delegate:anObject
                   event:theEvent];

    return;
}


- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj
               delegate:(id)anObject start:(int)selStart length:(int)selLength
{
    // Make sure that the frame is the text field rect and pass it up to the superclass
    [super selectWithFrame:[self textFieldRectFromRect:aRect]
                    inView:controlView
                    editor:textObj
                  delegate:anObject
                     start:selStart
                    length:selLength];

    return;
}


- (void)cancelButtonClickedWithFrame:(NSRect)aFrame inView:(NSView *)aView
{
    [self setStringValue:@""];
    _shouldShowCancelButton = NO;
    _shouldShowSelectedPopUpItem = YES;

    return;
}


- (void)popUpButtonClickedWithFrame:(NSRect)aFrame inView:(NSView *)aView
{
    aFrame.origin.y += 4.0;

    _shouldShowSelectedPopUpItem = NO;
    [popUpButtonCell performClickWithFrame:aFrame inView:aView];

    return;
}


- (void)searchSubmittedFromView:(NSView *)controlView
{
    _shouldShowCancelButton = [[self stringValue] isEqualToString:@""] ? NO : YES;

    [controlView setNeedsDisplay:YES];
    
    return;
}


// Returns a path around the widget
- (NSBezierPath *)fieldOutlinePathForRect:(NSRect)aRect
{
    NSBezierPath *fieldOutlinePath = [NSBezierPath bezierPath];
    NSPoint aPoint;

    // Start at the top left of the middle section
    aPoint.x = aRect.origin.x + [_leftImage size].width;
    aPoint.y = NSMaxY(aRect) - 1.0;
    [fieldOutlinePath moveToPoint:aPoint];

    // Trace to the top right of the middle section
    aPoint.x = NSMaxX(aRect) - [_rightImage size].width;
    [fieldOutlinePath lineToPoint:aPoint];

    // Trace around the right curve to the bottom right of the middle section
    aPoint.y = aRect.origin.y;
    [fieldOutlinePath curveToPoint:aPoint
                     controlPoint1:NSMakePoint(aPoint.x + 13.0, NSMaxY(aRect))
                     controlPoint2:NSMakePoint(aPoint.x + 13.0, aPoint.y)];

    // Trace to the bottom left of the middle section
    aPoint.x = aRect.origin.x + [_leftImage size].width;
    [fieldOutlinePath lineToPoint:aPoint];

    // Trace around the left curve to the top left of the middle section, i.e. the beginning
    aPoint.y = NSMaxY(aRect) - 1.0;
    [fieldOutlinePath curveToPoint:aPoint
                     controlPoint1:NSMakePoint(aPoint.x - 11.0, aRect.origin.y)
                     controlPoint2:NSMakePoint(aPoint.x - 11.0, aPoint.y)];

    [fieldOutlinePath closePath];

    return fieldOutlinePath;
}


// Returns a rect for where the cancel button should be
- (NSRect)cancelButtonRectFromRect:(NSRect)aRect
{
    // If we have a cancel button ready, return the rect for
    // where it should be. Else return an empty rect.
    if (_shouldShowCancelButton) {
        NSRect cancelButtonRect, remainderRect;
        float width = [_cancelImage size].width + symbolXOffset*2;

        NSDivideRect(aRect, &cancelButtonRect, &remainderRect, width, NSMaxXEdge);

        return cancelButtonRect;
    } else {
        return NSZeroRect;
    }
}

// Returns a rect for where the popUp button should be
- (NSRect)popUpButtonRectFromRect:(NSRect)aRect
{
    // If we are supposed to show the popUp button, return the
    // rect for where it should be. Else return an empty rect.
    if (hasPopUpButton) {
        NSRect popUpButtonRect, remainderRect;
        float width = symbolXOffset + [_popUpImage size].width + symbolXOffset;

        NSDivideRect(aRect, &popUpButtonRect, &remainderRect, width, NSMinXEdge);

        return popUpButtonRect;
    } else {
        return NSZeroRect;
    }
}

// Returns a rect for where the text field should be
- (NSRect)textFieldRectFromRect:(NSRect)aRect
{
    NSRect remainderRect, tempRect;
    float startWidth = hasPopUpButton ?
        [_popUpImage size].width + symbolXOffset : [_leftImage size].width;
    float endWidth = _shouldShowCancelButton ?
        [_cancelImage size].width : [_rightImage size].width;

    // Cut off the cancel button
    NSDivideRect(aRect, &tempRect, &remainderRect, endWidth, NSMaxXEdge);

    // Cut off the popUp button
    NSDivideRect(remainderRect, &tempRect, &remainderRect, startWidth, NSMinXEdge);

    // Cut a little of the bottom so that the text is centered well
    NSDivideRect(remainderRect, &tempRect, &remainderRect, (float)3, NSMinYEdge);

    return remainderRect;
}

@end

@implementation SearchTextFieldCell(PrivateMethods)

// If any of the static cached images is nil, loads them.  Returns YES if
// all were loaded.  Returns NO if any of the cached images is still nil.
- (BOOL)_loadImagesIfNecessary
{
    if (cachedLeftImage == nil) {
        cachedLeftImage = [[NSImage imageNamed:@"SearchLeft"] retain];
    }

    if (cachedMiddleImage == nil) {
        cachedMiddleImage = [[NSImage imageNamed:@"SearchMiddle"] retain];
        [cachedMiddleImage setScalesWhenResized:TRUE];
    }

    if (cachedRightImage == nil) {
        cachedRightImage = [[NSImage imageNamed:@"SearchRight"] retain];
    }

    if (cachedPopUpImage == nil) {
        cachedPopUpImage = [[NSImage imageNamed:@"SearchPopUp"] retain];
    }

    if (cachedCancelImage == nil) {
        cachedCancelImage = [[NSImage imageNamed:@"SearchCancel"] retain];
    }

    // Return YES if none of the cached images point to nil, else return NO.
    return (cachedLeftImage && cachedMiddleImage &&
            cachedRightImage && cachedCancelImage) ? YES : NO;
}


- (void)_popUpButtonSelectionChanged:(NSNotification *)aNotification
{
    return;
}


@end